Package: wru
Title: Who are You? Bayesian Prediction of Racial Category Using
        Surname, First Name, Middle Name, and Geolocation
Version: 3.1.0
Date: 2026-04-30
Authors@R: c(
    person("Kabir", "Khanna", , "kabirkhanna@gmail.com", role = "aut"),
    person("Brandon", "Bertelsen", , "brandon@bertelsen.ca", role = c("aut", "cre")),
    person("Santiago", "Olivella", , "olivella@unc.edu", role = "aut"),
    person("Evan", "Rosenman", , "etrrosenman@gmail.com", role = "aut"),
    person("Alexander", "Rossell Hayes", , "alexander@rossellhayes.com", role = "aut"),
    person("Kosuke", "Imai", , "imai@harvard.edu", role = "aut"),
    person("Noah", "Dasanaike", , "noahdasanaike@g.harvard.edu", role = "aut")
  )
Description: Predicts individual race/ethnicity using surname, first name,
    middle name, geolocation, and other attributes, such as gender and
    age. The method utilizes Bayes' Rule (with optional measurement error
    correction) to compute the posterior probability of each racial
    category for any given individual. The package implements methods
    described in Imai and Khanna (2016) "Improving Ecological Inference by
    Predicting Individual Ethnicity from Voter Registration Records"
    Political Analysis <DOI:10.1093/pan/mpw001> and Imai, Olivella, and
    Rosenman (2022) "Addressing census data problems in race imputation
    via fully Bayesian Improved Surname Geocoding and name supplements"
    <DOI:10.1126/sciadv.adc9824>.  The package also incorporates the data
    described in Rosenman, Olivella, and Imai (2023) "Race and ethnicity
    data for first, middle, and surnames"
    <DOI:10.1038/s41597-023-02202-2>.
License: GPL (>= 3)
URL: https://github.com/kosukeimai/wru
BugReports: https://github.com/kosukeimai/wru/issues
Depends: R (>= 4.1.0), utils
Imports: cli, dplyr, furrr, piggyback (>= 0.1.4), PL94171, purrr, Rcpp,
        rlang
Suggests: covr, future, reticulate, testthat (>= 3.0.0), tidycensus,
        tidyr
LinkingTo: Rcpp, RcppArmadillo
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: yes
LazyDataCompression: xz
LazyLoad: yes
RoxygenNote: 7.3.1
NeedsCompilation: yes
Packaged: 2026-05-07 03:49:58 UTC; b
Author: Kabir Khanna [aut],
  Brandon Bertelsen [aut, cre],
  Santiago Olivella [aut],
  Evan Rosenman [aut],
  Alexander Rossell Hayes [aut],
  Kosuke Imai [aut],
  Noah Dasanaike [aut]
Maintainer: Brandon Bertelsen <brandon@bertelsen.ca>
Repository: CRAN
Date/Publication: 2026-05-07 11:40:12 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-07 12:55:10 UTC; unix
Archs: wru.so.dSYM
