# Generated by roxygen2: do not edit by hand

export(as_fips_code)
export(as_state_abbreviation)
export(format_legacy_data)
export(get_census_data)
export(predict_race)
export(setup_ebisg)
import(PL94171)
importFrom(Rcpp,evalCpp)
importFrom(dplyr,coalesce)
importFrom(dplyr,pull)
importFrom(furrr,future_map_dfr)
importFrom(piggyback,pb_download)
importFrom(purrr,map_dfr)
importFrom(rlang,"%||%")
importFrom(stats,rmultinom)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(wru, .registration=TRUE)
