# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Collapsed Gibbs sampler for hWRU. Internal function
#'
#' @param last_name Integer vector of last name identifiers for each record (zero indexed; as all that follow). Must match columns numbers in M_rs.  
#' @param first_name See last_name
#' @param mid_name See last_name
#' @param geo Integer vector of geographic units for each record. Must match column number in N_rg
#' @param N_rg Integer matrix of race | geography counts in census (geograpgies in columns).
#' @param pi_s Numeric matrix of race | surname prior probabilities.
#' @param pi_f Same as `pi_s`, but for first names.
#' @param pi_m Same as `pi_s`, but for middle names.
#' @param pi_nr Matrix of marginal probability distribution over missing names; non-keyword names default to this distribution.
#' @param which_names Integer; 0=surname only. 1=surname + first name. 2= surname, first, and middle names.
#' @param samples Integer number of samples to take after (in total)
#' @param burnin Integer number of samples to discard as burn-in of Markov chain
#' @param race_init Integer vector of initial race assignments
#' @param verbose Boolean; should informative messages be printed?
#'
#' @keywords internal
sample_me <- function(last_name, first_name, mid_name, geo, N_rg, pi_s, pi_f, pi_m, pi_nr, which_names, samples, burnin, race_init, verbose) {
    .Call(`_wru_sample_me`, last_name, first_name, mid_name, geo, N_rg, pi_s, pi_f, pi_m, pi_nr, which_names, samples, burnin, race_init, verbose)
}

