#' Census Surname List (2000).
#'
#' Census Surname List from 2000 with race/ethnicity probabilities by surname.
#'
#' @format A data frame with 157,728 rows and 6 variables:
#' \describe{
#'   \item{surname}{Surname}
#'   \item{p_whi}{Pr(White | Surname)}
#'   \item{p_bla}{Pr(Black | Surname)}
#'   \item{p_his}{Pr(Hispanic/Latino | Surname)}
#'   \item{p_asi}{Pr(Asian/Pacific Islander | Surname)}
#'   \item{p_oth}{Pr(Other | Surname)}
#'   #' }
#'
#' @docType data
#' @keywords datasets
#' @name surnames2000
#' @examples
#' data(surnames2000)
"surnames2000"
