% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ebisg_utils.R
\name{ebisg_embed_names}
\alias{ebisg_embed_names}
\title{Embed names using a sentence-transformer model via Python}
\usage{
ebisg_embed_names(
  names,
  transformer = "intfloat/multilingual-e5-large",
  cache_dir = NULL
)
}
\arguments{
\item{names}{Character vector of names to embed.}

\item{transformer}{HuggingFace model ID (e.g.,
\code{"intfloat/multilingual-e5-large"}).}

\item{cache_dir}{Optional cache directory for model weights.}
}
\value{
A matrix of dimensions (length(names), embedding_dim).
}
\description{
Embed names using a sentence-transformer model via Python
}
\keyword{internal}
