% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ebisg_utils.R
\name{ebisg}
\alias{ebisg}
\title{Utilities for eBISG (embedding-supplemented BISG)}
\description{
These functions support the eBISG model, which uses pre-trained text
embeddings to predict race probabilities for surnames not found in
Census surname lists.
}
