% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ebisg_utils.R
\name{resolve_ebisg_model}
\alias{resolve_ebisg_model}
\title{Resolve an ebisg.model argument into a config list}
\usage{
resolve_ebisg_model(ebisg.model)
}
\arguments{
\item{ebisg.model}{A character string giving the HuggingFace model ID
of a built-in model (e.g., \code{"intfloat/multilingual-e5-large"}),
or a named list with elements \code{transformer}, \code{dim},
\code{surname_mlp}, and optionally \code{firstname_mlp}.}
}
\value{
A list with elements \code{transformer}, \code{dim},
\code{surname_mlp}, \code{firstname_mlp}, \code{fullname_mlp}, \code{tag}.
}
\description{
Resolve an ebisg.model argument into a config list
}
\keyword{internal}
